/**
** This software is distributed under the terms of the General Public License.  
** 
** Program : BuildIT!
** Version : 0.2
** Authors : Artur Prusakiewicz <prusak@mat.uni.torun.pl>
**	     Michal Switala <scisly@mat.uni.torun.pl>
** E-MAIL  : build@mat.uni.torun.pl
** WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
**           http://www.mat.uni.torun.pl/~prusak/buildit/
**/
#ifndef _TREE_H
#define _TREE_H

#include "build.h"
#include <qstring.h>
#include <qfile.h>
#include <qdir.h>

/*!
  \class MyTree
  \brief MyTree class is the internal structure of components in created project

  MyTree class is the internal structure of components in the form of Tree. It is used by other classes to set or get components parameters.
*/

class MyTree
{
 public:  
  MyTree();
  ~MyTree();
  
  /*!
    Return pointer to component with name tag identify with \e id
  */
  MyTree* Find(QString* id);
  
  /*! 
    Return pointer to parent of component's with name tag identify with \e id
  */
  MyTree* FindParent(QString* id);
  
  /*!
    Return first found pointer to component which has \e what as item \e which in array of FEATURES1 <br>
    Function scans Tree starts from \e t and then scans children
  */
  MyTree* findFeature1(MyTree *t, int which, QString *what);

  /*!
    Write Tree structure to file \e filename where \e home is a user directory
  */
  bool Tree2File(QString filename, QString home);

  /*!
    Make Tree structure from file \e filename where \e home is a user directory
  */
  bool File2Tree(QString filename, QString home);

  /*!
    Add new component to Tree <br>
    tab1[] - array of FEATURES1 (strings) <br>
    tab2[] - array of FEATURES2 (integer) <br>
    Function knows where to add component because of id written in tab1[0]
  */
  bool Add(QString* tab1[], int tab2[]);

  /*!
    Remove component with FEATURE1[0]=\e id from Tree structure
  */
  bool Remove(QString* id);

  /*!
    Update component identify with tab1[0] <br>
    tab1[] - array of FEATURES1 (strings) <br>
    tab2[] - array of FEATURES2 (integer) <br>
  */
  bool Update(QString* tab1[], int tab2[]);

  /*!
    Return FEATURE1[\e which] for called MyTree object
  */
  QString* GetFeature1(int which);

  /*!
    Return FEATURE2[\e which] for called MyTree object
  */
  int GetFeature2(int which);

  /*!
    Set FEATURE1[\e which] with value what for called MyTree object
  */
  bool SetFeature1(int which, QString* what);

  /*!
    Set FEATURE1[\e which] with value what for called MyTree object
  */
  bool SetFeature1(int which, QString what);

  /*!
    Set FEATURE2[\e which] with value what for called MyTree object
  */
  bool SetFeature2(int which, int what);

  /*!
    Increment number of children for called MyTree object
  */
  bool IncChildren();

  /*!
    Decrement number of children for called MyTree object
  */
  bool DecChildren();

  /*!
    Return pointer to parent component for called MyTree object
  */
  MyTree* GetParent() { return parent; }

  /*!
    Return one's pointer for called MyTree object
  */
  MyTree* GetPtr() { return ptr; }

  /*! 
    Return pointer to \e which children for called MyTree object
  */
  MyTree* GetChildren(int which) { return children[which]; }
  
  /*!
    Return number of children in first generation for called MyTree object
  */
  int GetNumChildren() { return numchildren; }

  /*!
    Return number of all children for called MyTree object
  */
  int GetNumAllChildren();

  /*!
    Return in \e result[] array of FEATURE1[\e what] with \n of items which have FEATURE1[\e where]=\e name
  */
  bool FindFeature(MyTree *t, int where, QString name, int what, QString *result[], int &n);

  bool getItems(QString *result);
  bool getSub(MyTree *t,QString *result, int &n);
  bool countSub(MyTree *t,int &n);

  void getXY(QString ID,int &x,int &y);
  bool countFeature(MyTree *t,int which, int &sum);

  /*!
    \internal
  */
  bool Show(MyTree *p);
  
 private:
  /*!
    Cut string \e base with id on parts and return them in \e result[] array <br>
    i.e string "1|3|4" return: <br>
    result[0] = "1" <br>
    result[1] = "3" <br>
    result[2] = "4" 
  */
  int CutIdString(QString* base, QString* result[]);

  /*!
    Cut \e buf (which is the line from save file) to arrays of features and write result to \e t1 and \t2
  */
  bool CutFileData(QString buf, QString *t1[], int t2[]);

  /*!
    \internal
  */
  bool RemoveLeaf(MyTree *p);

  /*!
    \internal
  */
  bool WriteLeaf(MyTree *p, QFile *f);

  /*!
    Array of strings which describe feature of component. The size is NUMFEATURE1
  */
  QString* feature1[NUMFEATURE1];
  
  /*!
    Array of integers which describe feature of component. The size is NUMFEATURE2
  */
  int feature2[NUMFEATURE2];

  /*!
    root = pointer to Root component in the Tree (only for first component in Tree)
    root = NULL for other components
  */
  MyTree* root;
  
  /*!
    Pointer to parent component for current MyTree object
  */
  MyTree *parent;
  
  /*!
    Pointer to itself
  */
  MyTree* ptr;

  /*!
    Array of pointer to component's children
  */
  MyTree *children[MAXCHILDREN];

  /*!
    Number of component's children 
  */
  int numchildren;
  /*!
    \internal
  */
  int terror;
 };

#endif
